.openweather-button {
border-radius: 32px;
padding: 13px;
}

.openweather-button:hover{
border: none;
padding: 14px;
}

.openweather-button > StIcon {
icon-size: 16px;
}

.openweather-button > StLabel {
padding:6px;
}

.openweather-menu-button-container{
padding: 0px;
}

.openweather-button-box{
padding: 0px;
spacing: 12px;
}
.openweather-button-box-38:ltr{
padding-left: 24px;
}

.openweather-button-box-38:rtl{
padding-right: 24px;
}

.openweather-provider {
border-width: 0px;
border-radius:5px;
padding:5px;
}

.openweather-current-summarybox,
.openweather-forecast-icon,
.openweather-current-databox-captions,
.openweather-current-databox-values,
.openweather-current-icon,
.openweather-forecast-summary,
.openweather-forecast-temperature{
border-radius: 0px;
padding: 0px;
border: 0px;
}

.openweather-current-summarybox {
padding-top:5px;
}

.openweather-current-summary {
font-size: 31px;
}

.openweather-current-databox {
padding: 5px 15px 0px;
}

.openweather-current-icon {
padding-right: 30px;
}

.openweather-current-iconbox {
padding-right: 20px;
}

.openweather-icon:ltr {
padding-right: 5px;
}

.openweather-icon:rtl {
padding-left: 5px;
}

.openweather-current-databox-captions {
text-align: right;
padding-right: 5px;
spacing: 0;
}

.openweather-forecast-icon {
padding-right: 15px;
}

.openweather-forecasts {
-st-hfade-offset: 60px;
}

.openweather-forecast-databox {
min-width: 160px;
}

.openweather-forecast-day {
font-size: 90%;
spacing: 0;
}

.openweather-forecast-iconbox {
vertical-align: top;
}

.openweather-sunrise-icon {
padding-right: 5px;
}

.openweather-sunset-icon {
padding-left: 15px;
padding-right: 5px;
}

.openweather-build-icon {
padding-left: 15px;
padding-right: 5px;
}
